import glob

from setuptools import setup, find_packages
import os

# 读取 requirements.txt 文件内容
def read_requirements():
    req_path = "requirements.txt"
    if not os.path.exists(req_path):
        return []
    with open(req_path, encoding="utf-8") as f:
        return [line.strip() for line in f if line.strip() and not line.startswith("#")]


# 找出所有根目录下的.py文件（不包括__init__.py）
def find_py_modules():
    return [os.path.splitext(os.path.basename(f))[0]
            for f in glob.glob("*.py")
            if os.path.isfile(f) and not f.startswith('__init__')]


setup(
    name="crossborder",
    version="0.1.0",
    packages=find_packages(where="."),
    package_dir={"": "."},

    # 添加根目录下的.py文件
    py_modules=find_py_modules(),

    include_package_data=True,
    install_requires=read_requirements(),
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'run-shandong=shandong.selenium_shandong_download:main',
            'run-guangdong=guangdong.selenium_guangdong_download:main',
            'run-guangdong-city=guangdong.selenium_guangdong_city:main',
            'run-henan=henan.selenium_henan_download:main',
            'run-fujian=fujian.selenium_fujian_download.py:main',
            'run-anhui=anhui.crawl_gov_anhui_full:main',
            'run-jiangsu=jiangsu.gov_commodity_jiangsu_country:main',
            'run-hebei=hebei.crawl_gov_hebei_full:main',
            'run-zhejiang=zhejiang.crawl_gov_zhejiang_full:main',
            'run-quanguo=quanguo.selenium_download:main',

            # 统一入口命令
            'run-crossborder=crossborder.cli:main',
        ],
    },
)
